﻿using System;
using System.Threading;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using SecretLabs.NETMF.Hardware;
using SecretLabs.NETMF.Hardware.NetduinoMini;
using System.IO.Ports;      // Microsoft.SPOT.Hardware.SerialPort
using System.Text;          // Encoding.UTF8.GetBytes(str);


namespace BoeBotLib
{

    public class CPU
    {

        public static Cpu.Pin P0 = (Cpu.Pin)Pins.GPIO_PIN_5;
        public static Cpu.Pin P1 = (Cpu.Pin)Pins.GPIO_PIN_6;
        public static Cpu.Pin P2 = (Cpu.Pin)Pins.GPIO_PIN_7;
        public static Cpu.Pin P3 = (Cpu.Pin)Pins.GPIO_PIN_8;
        public static Cpu.Pin P4 = (Cpu.Pin)Pins.GPIO_PIN_9;
        public static Cpu.Pin P5 = (Cpu.Pin)Pins.GPIO_PIN_10;
        public static Cpu.Pin P6 = (Cpu.Pin)Pins.GPIO_PIN_11;
        public static Cpu.Pin P7 = (Cpu.Pin)Pins.GPIO_PIN_12;
        public static Cpu.Pin P8 = (Cpu.Pin)Pins.GPIO_PIN_13;
        public static Cpu.Pin P9 = (Cpu.Pin)Pins.GPIO_PIN_14;
        public static Cpu.Pin P10 = (Cpu.Pin)Pins.GPIO_PIN_15;
        public static Cpu.Pin P11 = (Cpu.Pin)Pins.GPIO_PIN_16;
        public static Cpu.Pin P12 = (Cpu.Pin)Pins.GPIO_PIN_17;
        public static Cpu.Pin P13 = (Cpu.Pin)Pins.GPIO_PIN_18;
        public static Cpu.Pin P14 = (Cpu.Pin)Pins.GPIO_PIN_19;
        public static Cpu.Pin P15 = (Cpu.Pin)Pins.GPIO_PIN_20;

        
        public static void delay(int n)
        {
            Thread.Sleep(n);
        }

        static inArray[] Din = new inArray[4];// 注意
        static int usePin = 0;
        public static void setInput(Cpu.Pin pin)
        {
            InputPort port = new InputPort(pin, false, Port.ResistorMode.PullUp);
            Din[usePin].inp = port;
            Din[usePin++].pin = pin;
        }
        public static bool readPin(Cpu.Pin pin)
        {
            int i;
            for (i = 0; i < usePin; i++)
            {
                if (Din[i].pin == pin) break;
            }
            if (i == usePin) return false;// exception
            return Din[i].inp.Read();           
        }
        static outArray[] Dout = new outArray[4];// 注意
        static int usePout = 0;
        public static void setOutput(Cpu.Pin pin)
        {
            OutputPort port = new OutputPort(pin, false);
            Dout[usePout].outp = port;
            Dout[usePout++].pin = pin;
        }
        public static bool writePin(Cpu.Pin pin, bool val)
        {
            int i;
            for (i = 0; i < usePout; i++)
            {
                if (Dout[i].pin == pin) break;
            }
            if (i == usePout) return false;// exception
            Dout[i].outp.Write(val);
            return true;
        }
    }

    struct inArray
    {
        public InputPort inp;
        public Cpu.Pin  pin;
    }
    struct outArray
    {
        public OutputPort outp;
        public Cpu.Pin pin;
    }

    public class ServoMotor : IDisposable
    {
        PWM pwm;
        uint period, high;
        bool stopped;

        public void Dispose()
        {
            pwm.Dispose();
        }

        public ServoMotor(Cpu.Pin pin)
        {
            pwm = new PWM(pin);
            stopped = true;
            //stop();
        }

        public void SetSpeed(uint highTime)
        {
            pwm.SetPulse(20000, highTime);     // 周期:20ms
        }
        public void move(int speed, int forward, int center, int backward)
        {
            uint t = (uint)((((speed > 0) ? forward : backward) * speed) / 100 + center);

        }
        public void update(uint highTime, uint lowTime)
        {
            period = highTime + lowTime;
            high = highTime;
            if(!stopped) pwm.SetPulse(period, high);
        }
        public void start()
        {
            pwm.SetPulse(period, high);
            stopped = false;
        }

        public void stop()
        {
            pwm.SetPulse(20000, 0);
            stopped = true;
        }

    }

    public class ADcon : IDisposable
    {
        AnalogInput adc;
        public int Value {
            get
            {
                return adc.Read();
            } 
        }
        public void Dispose()
        {
            adc.Dispose();
        }

        public ADcon(Cpu.Pin pin)
        {
            adc = new AnalogInput(pin);
        }
        public int GetValue()
        {
            return adc.Read();
        }
    }
/// <summary>
/// 単位は0.1μ秒となる
/// </summary>
    public class _Timer
    {
        DateTime markTime;
        public _Timer()
        {
            mark();
        }

        public void mark()
        {
            markTime = DateTime.Now;
        }
        public bool timeout(int hi, int lo)
        {
            if (hi <= (DateTime.Now - markTime).Ticks) return true;
            return false;
        }
        public bool timeout(int hi)
        {
            if (hi <= (DateTime.Now - markTime).Ticks) return true;
            return false;
        }
        public bool timeout(long hi)
        {
            if (hi <= (DateTime.Now - markTime).Ticks) return true;
            return false;
        }
        public long Time
        {
            get
            {
                return (DateTime.Now - markTime).Ticks;
            }
        }
    }
}
